<?php

namespace wisdmlabs\cpb;

/**
* This class handles the CPB Product type settings and the data related to each of * the CPB products.
*/
class WdmProductBundleType
{
    /**
    * If the user is admin add the following actions :
    * 1: Action to add CPB Product type in the list of product types.
    * 2: Action to add the custom settings for CPB Product type.
    * 3: Action to add the tab for CPB Product settings.
    * 4: Action to save the bundled product data in DB.
    * 5: Action to add-meta boxes for CPB Products on Product edit/create page.
    * 6: Action to check if the prefilled product is a sold individually product.
    * 7: Filter for the json search of products for add-on products to show only simple products.
    * 8: Action to enqueue admin scripts and styles.
    */
    public function __construct()
    {
        if (is_admin()) {
            add_filter('product_type_selector', array($this, 'wdmAddCustomProductType'), 10, 1);
            add_filter('product_type_options', array($this, 'wdmAddEnableSubscriptionSetting'), 10, 1);
            // add_action('woocommerce_product_options_general_product_data', array($this, 'wdmAddCustomSettings'), 99);
            add_action('woocommerce_product_write_panel_tabs', array($this, 'woocommerceProductWritePanelTabsFunc'));
            add_action('woocommerce_process_product_meta_wdm_bundle_product', array($this, 'woocommerceProcessProductMetaBundleFunc'), 100);

            add_action('woocommerce_product_data_panels', array($this, 'woocommerceProductWritePanelsFunc'));
            add_filter('woocommerce_product_data_tabs', array($this, 'woocommerceProductGeneralTab'), 10, 1);

            add_action('wp_ajax_wdm_is_sold_individual', array($this, 'checkIfSoldIndividual'));

            add_action('admin_enqueue_scripts', array($this, 'adminEnqueueScripts'));
        }
    }

    public function woocommerceProductGeneralTab($tabs)
    {
        $tabs['general']['class'][] = "show_if_wdm_bundle_product";

        return $tabs;
    }

    /**
    * If the current screen is edit product or product screen enqueue the required * scripts.
    * Prepare the data for localization.
    */
    public function adminEnqueueScripts()
    {
        global $post, $woo_wdm_bundle;
        $screen       = get_current_screen();
        $screen_id    = $screen ? $screen->id : '';
        $post_id = isset($post->ID) ? $post->ID : 0;
        if (in_array($screen_id, array('product', 'edit-product'))) {
            $layout = $woo_wdm_bundle->getDesktopLayout($post_id);
            $selectedLayout = basename($layout);
            wp_enqueue_script('wdm-publish-admin-hook');
            wp_enqueue_script('wdm-cpb-settings-js');
            wp_enqueue_script('wdm-alpha-color-picker');
            wp_localize_script(
                'wdm-cpb-settings-js',
                'cpb_layout_object',
                array(
                    'ajax_url'  => admin_url('admin-ajax.php'),
                    'selectedLayout'    => $selectedLayout
                )
            );
        }
    }

    /**
    * Add the product type of Custom Product Box in product types array of
    * woocommerce.
    * @param array $product_types default product types in woocommerce.
    * @return array $product_types product types with additional custom product
    * type
    */
    public function wdmAddCustomProductType($product_types)
    {
        $product_types['wdm_bundle_product'] = __(
            'Custom Product Box',
            'custom-product-boxes'
        );
        wp_enqueue_script('wdm-publish-admin-hook');
        return $product_types;
    }

    /**
     * Checks whether the products are sold individually & have prefilled
     * quantity set greater than 1. Invoked by ajax call
     * return to ajax json array returns array containing produt ids of sold
     * individual products whoes quantity is set greater than 1
     */
    public function checkIfSoldIndividual()
    {
        if (!isset($_POST['product_ids'])) {
            return;
        }

        $productIds = $_POST['product_ids'];
        $productData = $_POST['product_data'];
        $parentIds = $_POST['parent_ids'];
        $sldindIds = array();
        foreach ($productIds as $productId => $qty) {
            $product = wc_get_product($productData[$productId]);
            if ($product->is_sold_individually() && $qty > 1) {
                $sldindIds[] = $productData[$productId];
            }
        }
        $sldindIds = array_unique($sldindIds);
        echo json_encode($sldindIds);
        die();
    }

    /**
    * For the custom product box product type.
    * Add the custom settings for regular and sale price(Can be used for base
    * price of CPB)
    * If the tax is enabled add the settings of tax status and tax class for CPB
    * Product type.
    */
    public function wdmAddEnableSubscriptionSetting($product_type_options)
    {
        if (!isSubscriptionActive()) {
            return $product_type_options;
        }

        $product_type_options['wdm_enable_product_subscription'] =  array(
            'id'            => 'wdm_enable_product_subscription',
            'wrapper_class' => 'show_if_wdm_bundle_product',
            'label'         => __('Enable subscription', 'custom-product-boxes'),
            'description'   => __('Allows admin to enable subscription for this specific product.', 'custom-product-boxes'),
            'default'       => esc_attr(get_post_meta(get_the_ID(), 'wdm_enable_product_subscription', true))
        );
        
        return $product_type_options;
    }

    /**
    * Add the Custom Box settings tab when product type selected is CPB.
    */
    public function woocommerceProductWritePanelTabsFunc()
    {
        echo '<li class="wdm_bundle_product_tab show_if_wdm_bundle_product wdm_bundle_product_options linked_product_options"><a href="#wdm_bundle_product_data">' . __('Custom Box Settings', 'custom-product-boxes') . '</a></li>';
    }

    /**
    * Adds the meta-boxes for the CPB Settings on the product edit/create page.
    * only if CPB Product type is selected.
    * Removes the deleted products data from the CPB Product.
    * Get the existing pre-filled products for the current CPB Product.
    */
    public function woocommerceProductWritePanelsFunc()
    {
        global $post, $woo_wdm_bundle, $prefillManager;

        wp_nonce_field('wdm_cpb_nonce', 'wdm_cpb_nonce');

        $cpbProductID = $post->ID;
        $cpbProduct = wc_get_product($post->ID);

        $allLayouts = WdmCPBLayoutSetting::getAllLayouts();
        //Bundled products
        $product_field_types = maybe_unserialize(
            get_post_meta(
                $post->ID,
                '_bundle_data',
                true
            )
        );

        if (is_array($product_field_types) && !empty($product_field_types)) {
            $product_field_types = unlinkDeletedProducts($product_field_types, $post->ID);
            $bundleData = array_values($product_field_types);
            $prefillData = getPrefillBundleForVariableProduct($product_field_types);
            $prefillBundleData = json_encode($prefillData);
            $bundleData = json_encode($bundleData);
            wp_localize_script('wdm-publish-admin-hook', 'cpb_bundled_data', array(
                'bundled_data'              => $bundleData,
                'isProductCPBSubscription'  => isProductCPBSubscription($cpbProductID),
            ));

            wp_localize_script('wdm-prefilled-products-cpb', 'cpb_bundled_prefill', array(
                'prefillBundleData' => $prefillBundleData,
            ));
        } else {
            wp_localize_script('wdm-publish-admin-hook', 'cpb_bundled_data', array());

            wp_localize_script('wdm-prefilled-products-cpb', 'cpb_bundled_prefill', array());
        }

        // json ids
        $json_ids = array();

        if (is_array($product_field_types)) {
            foreach ($product_field_types as $uniqueID => $product_data) {
                $product_id = isset($product_data['variation_id']) ? $product_data['variation_id'] : $product_data['product_id'];
                $product = wc_get_product($product_id);
                $json_ids[$product_id] = wp_kses_post($product->get_formatted_name());
                unset($product_data);
            }
        }

        // preparing variables to use in the HTML we are parsing
        $product_box_label = __('Product Box Settings', 'custom-product-boxes');
        $json_ids_imploded = implode(',', array_keys($json_ids));
        $json_ids_json = esc_attr(json_encode($json_ids));

        $helptip_image = WC()->plugin_url().'/assets/images/help.png';
        $helptip = __('Select the products which can be added to the Custom Product Box. Kindly select more than one product. If you add main variable product as an add-on then all its variations will be included as CPB add-ons', 'custom-product-boxes');
        $addon_products_label = __('Add-On Products', 'custom-product-boxes');

        echo "<div id='wdm_bundle_product_data' class='panel woocommerce_options_panel'>
              <div class='options_group'>
              <div class='wc-wdm_bundle_products'>
              <div class='wdm_bundle_products_info'>
                {$product_box_label}
              </div>";

        echo '<div class="options_group show_if_wdm_bundle_product">';

        woocommerce_wp_select(
            array(
            'id' => '_wdm_cpb_pricing_type_field',
            'label' => __('Pricing Type', 'custom-product-boxes'),
            'desc_tip' => 'true',
            'description' => __('Select the pricing type.', 'custom-product-boxes'),
            'options' => array(
            'wdm-cpb-fixed-pp' => __('Fixed Pricing', 'custom-product-boxes'),
            'wdm-cpb-dynamic-pp-base' => __('Per Product Pricing with Base Price', 'custom-product-boxes'),
            'wdm-cpb-dynamic-pp-nobase' => __('Per Product Pricing without Base Price', 'custom-product-boxes')
            ),
            )
        );

        woocommerce_wp_text_input(
            array(
            'id' => '_wdm_grid_field',
            'label' => __('Box Quantity', 'custom-product-boxes'),
            'placeholder' => '',
            'desc_tip' => 'true',
            'description' => __('Set the number of items which can be added to the box.', 'custom-product-boxes'),
            'type' => 'number',
            'custom_attributes' => array(
            'step' => 'any',
            'min' => '2',
            ),
            'value' => get_post_meta($post->ID, '_wdm_grid_field', true),
            )
        );

        woocommerce_wp_select(
            array(
            'id' => '_wdm_desktop_layout',
            'label' => __('Select box layout', 'custom-product-boxes'),
            'desc_tip' => 'true',
            'description' => __('Select the box layout. You can also create a custom layout by overriding our default template. To know more, please refer our user guide.', 'custom-product-boxes'),
            'options' => $allLayouts,
            'value' => $woo_wdm_bundle->getDesktopLayout($post->ID),
            )
        );

        woocommerce_wp_select(
            array(
            'id' => '_wdm_column_field',
            'label' => __('Columns in Gift Box', 'custom-product-boxes'),
            'desc_tip' => 'true',
            'description' => __("Select the number of columns for the gift layout (This will affect the display of the layout on your custom box page).", 'custom-product-boxes'),
            'options' => array(
                    'wdm-bundle-single-product-col-2' => __('2', 'custom-product-boxes'),
                    'wdm-bundle-single-product-col-3' => __('3', 'custom-product-boxes'),
                ),
            )
        );
        woocommerce_wp_select(
            array(
            'id' => '_wdm_item_field',
            'label' => __('Items per row in Gift Box (Recomended 8 for large box quantity).', 'custom-product-boxes'),
            'desc_tip' => 'true',
            'description' => __("Select the number of columns for the gift layout (This will affect the display of the layout on your custom box page).", 'custom-product-boxes'),
            'options' => array(
                    'wdm-bundle-single-product-col-4' => __('4', 'custom-product-boxes'),
                    'wdm-bundle-single-product-col-5' => __('5', 'custom-product-boxes'),
                    'wdm-bundle-single-product-col-6' => __('6', 'custom-product-boxes'),
                    'wdm-bundle-single-product-col-7' => __('7', 'custom-product-boxes'),
                    'wdm-bundle-single-product-col-8' => __('8', 'custom-product-boxes'),
                ),
            )
        );
        // }

        $boxSetColor = get_post_meta($post->ID, '_wdm_gift_boxes_color', true);
        $boxSetColor = isset($boxSetColor) && !empty($boxSetColor) ? $boxSetColor : "rgba(221,153,51,0.25)";

        woocommerce_wp_text_input(
            array(
            'id' => '_wdm_gift_boxes_color',
            'label' => __('Color for Gift boxes', 'custom-product-boxes'),
            'placeholder' => '',
            'desc_tip' => 'true',
            'description' => __('Set the alpha color for the gift boxes.', 'custom-product-boxes'),
            'type' => 'text',
            'value' => $boxSetColor,
            )
        );

        woocommerce_wp_text_input(
            array(
            'id' => '_wdm_gift_bgcolor',
            'label' => __('Background color for Gift box', 'custom-product-boxes'),
            'placeholder' => '',
            'desc_tip' => 'true',
            'description' => __('Set the alpha color for the background of gift boxes. This should match the themes color.', 'custom-product-boxes'),
            'type' => 'text',
            'value' => get_post_meta($post->ID, '_wdm_gift_bgcolor', true),
            )
        );

        woocommerce_wp_select(
            array(
            'id' => '_wdm_product_grid',
            'label' => __('Columns in Product Layout', 'custom-product-boxes'),
            'desc_tip' => 'true',
            'description' => __("Select the number of columns for the products layout (This will affect the display of the layout on your custom box page).", 'custom-product-boxes'),
            'options' => array(
                    'bundled_product-col-2' => __('2', 'custom-product-boxes'),
                    'bundled_product-col-3' => __('3', 'custom-product-boxes'),
                ),
            )
        );

        woocommerce_wp_select(
            array(
            'id' => '_wdm_product_item_grid',
            'label' => __('Items per row in Product Layout', 'custom-product-boxes'),
            'desc_tip' => 'true',
            'description' => __("Select the number of columns for the products layout (This will affect the display of the layout on your custom box page).", 'custom-product-boxes'),
            'options' => array(
                    'bundled_product-col-4' => __('4', 'custom-product-boxes'),
                    'bundled_product-col-5' => __('5', 'custom-product-boxes'),
                    'bundled_product-col-6' => __('6', 'custom-product-boxes'),
                    'bundled_product-col-7' => __('7', 'custom-product-boxes'),
                    'bundled_product-col-8' => __('8', 'custom-product-boxes'),
                ),
            )
        );

        woocommerce_wp_checkbox(
            array(
            'id' => 'wdm_boxes_selection',
            'label' => __('Allow Partially-Filled Box', 'custom-product-boxes'),
            'description' => __('Allow the purchase of box which has not been filled to its full capacity', 'custom-product-boxes'),
            'value' => esc_attr(get_post_meta($post->ID, 'wdm_boxes_selection', true)))
        );

        woocommerce_wp_checkbox(
            array(
            'id' => 'wdm_order_by_date',
            'label' => __('Sort Products by Date', 'custom-product-boxes'),
            'description' => __('Adds newly added product to the top', 'custom-product-boxes'),
            'value' => esc_attr(get_post_meta($post->ID, 'wdm_order_by_date', true)))
        );

        woocommerce_wp_checkbox(
            array(
            'id' => 'wdm_disable_scroll',
            'label' => __('Allow Scroll Lock', 'custom-product-boxes'),
            'description' => __('Enables the scroll of gift box', 'custom-product-boxes'),
            'value' => esc_attr(get_post_meta($post->ID, 'wdm_disable_scroll', true)))
        );

        woocommerce_wp_checkbox(
            array(
            'id' => '_wdm_enable_gift_message',
            'label' => __('Enable Gift Message', 'custom-product-boxes'),
            'desc_tip' => 'true',
            'description' => __("Allows Customers to send a message along with the Gift Box", 'custom-product-boxes'),
            'value' => esc_attr(get_post_meta($post->ID, '_wdm_enable_gift_message', true)))
        );

        woocommerce_wp_text_input(
            array(
            'id' => '_wdm_gift_message_label',
            'label' => __('Gift Message Label', 'custom-product-boxes'),
            'placeholder' => '',
            'desc_tip' => 'true',
            'description' => __("Set a label for 'Gift Message' field", 'custom-product-boxes'),
            'type' => 'text',
            )
        );

        echo '</div>';
        ?>
        <div class="wdm_bundle_products_selector">
        <?php
            $this->wdmSelectAddOnProduct($post, $addon_products_label, $json_ids, $json_ids_imploded, $json_ids_json, $helptip, $helptip_image);
        ?>
        </div>
        <div>
            <p class='form-field wdm_addon_note'>
                <label>
                    <?php _e('NOTE:', 'custom-product-boxes'); ?>
                </label>
                <ol class = 'wdm_note_list'>
                    <li>
                        <?php _e("When you add a main variable product to the Add-On Product list, the list includes all of its variations.", 'custom-product-boxes'); ?>
                    </li>
                    <li>
                        <?php _e("If you add a main variable product as well as its variation to the Add-On Product list, then the variation will be shown only once.", 'custom-product-boxes'); ?>
                    </li>
                    <li>
                        <?php _e("For prefilled products add quantity that is available in stock.", 'custom-product-boxes'); ?>
                    </li>
                    <li>
                        <?php _e("Once you update the individual product deatils you need to update the product again in the Add-On Product list.", 'custom-product-boxes'); ?>
                    </li>
                </ol>
            </p>
        </div>
                <?php
                $prefill_list = $prefillManager->getPrefilledProducts($post->ID);

                if (empty($prefill_list)) {
                    update_post_meta($post->ID, 'wdm_prefilled_box', 'no');
                }

                woocommerce_wp_checkbox(
                    array(
                        'id' => 'wdm_prefilled_box',
                        'label' => __('Pre-Filled Box', 'custom-product-boxes'),
                        'description' => __('Allow pre-filled box', 'custom-product-boxes'),
                        'value' => esc_attr(get_post_meta($post->ID, 'wdm_prefilled_box', true))
                    )
                );

                $mandatory = __("Mandatory", 'custom-product-boxes');
                $mandatoryTip = __("Check to make the selected pre-filled products mandatory in the custom box. (Products which are marked mandatory cannot be removed from the box)", 'custom-product-boxes');

                echo "<div class = 'prefill_div'>
                <table class = 'prefill_table' id = 'prefill_table_id'>
                <thead><tr><th><span class='help_tip tips' data-tip='" . esc_attr($mandatoryTip) . "'>" . esc_attr($mandatory) . "</span></th><th>".__("Product Name", 'custom-product-boxes')."</th><th>".__("Quantity", 'custom-product-boxes')."</th><th class = 'cpb_blank'></th></tr></thead>
                    <tbody>";

                if ($product_field_types) {
                    $this->wdmCpbPrefillTable($prefillData, $prefill_list);
                }

                echo "      </tbody>
                        </table>";

                woocommerce_wp_checkbox(
                    array(
                    'id' => 'wdm_swap_products',
                    'label' => __('Remove Mandatory Products', 'custom-product-boxes'),
                    'description' => __('Allows user to remove mandatory products only if they run out of stock', 'custom-product-boxes'),
                    'value' => esc_attr(get_post_meta($post->ID, 'wdm_swap_products', true)))
                );

                echo "</div>";

                echo "</div>
                </div>";

                    echo '</div>';

                    echo '<style>
                    /*.woocommerce_options_panel .wdm_bundle_products_selector .chosen-container-multi {
                        width : 90% !important;
                    }

                    .wdm_bundle_products_selector p.form-field.product_field_type{
                        padding : 0 !important;
                    }*/
                    .wdm_bundle_products_selector p.form-field .wc_products_selections {
                          width: 95% !important;
                          margin-top: 11px;
                    }

                    .wdm_bundle_products_selector p.form-field.wdm_product_selector {
                        padding-left: 16px !important;
                    }
                    .wc-wdm_bundle_products .wdm_bundle_products_info {
                        padding-left: 12px;
                        margin-top: 11px;
                    }
                    .wdm_bundle_products_selector .product_field_type_title{
                        padding-bottom: 6px;
                    }

                </style>';
    }

    /**
    * For the selection of the add-on products on the backend,we use the
    * woocommerce action of woocommerce_json_search_products for searching the
    * products.
    * Display the add-on product meta-box.
    * @param object $post Current product post.
    * @param string $addon_products_label Add-on product label.
    * @param array $json_ids array of product-id=>formatted name(name with the sku * or id)
    * @param array $json_ids_imploded product-ids array.
    * @param json array $json_ids_json json array  of product-id=>formatted name(
    * name with the sku or id)
    * @param string $helptip Helptip message.
    * @param string $helptip_image path to helptip icon image.
    */
    public function wdmSelectAddOnProduct($post, $addon_products_label, $json_ids, $json_ids_imploded, $json_ids_json, $helptip, $helptip_image)
    {
            ?>
                <p class='form-field wdm_product_selector'>
                    <span class='product_field_type_title'>
                        <?php echo $addon_products_label; ?>
                    </span>

                <input type='hidden' id='add_onProducts' name='add_onProducts' />
                <input type='hidden' id='product_field_type' name='product_field_type' class='wc_products_selections' style='width: 50%;' data-placeholder='Search for a product&hellip;' data-action='woocommerce_cpb_json_search_products_and_variations' data-multiple='true' data-selected="<?php echo $json_ids_json; ?>" value ="<?php echo $json_ids_imploded; ?>" data-excluded ="<?php echo $json_ids_json; ?>"/>

                <img class='help_tip' data-tip="<?php echo $helptip; ?>" src="<?php echo $helptip_image; ?>" height='16' width='16' />
                </p>
            <?php
    }


    /**
     * Generates the table structure for prefilled products saved in databse. i.e
     * loads the prefilled product in php
     * @param  array $product_field_types array of products bundled in CPB product
     * @param  array $prefill_list array of products to be prefilled
     */
    public function wdmCpbPrefillTable($product_field_types, $prefill_list)
    {
        global $post, $woo_wdm_bundle;
        $cpbKeys = array_keys($product_field_types);
        $maxVal = get_post_meta($post->ID, '_wdm_grid_field', true);
        $addImage = $woo_wdm_bundle->wooWdmBundlesPluginUrl() .'/assets/images/plus-icon.png';
        $removeImage = $woo_wdm_bundle->wooWdmBundlesPluginUrl() .'/assets/images/minus-icon.png';
        $i = 0;

        if (!empty($prefill_list)) {
            foreach ($prefill_list as $key) {
                echo "<tr>";
                echo $this->generateMandatoryCheckbox($cpbKeys, $key, $product_field_types);
                echo "<td><select name='wdm_cpb_products[]' class='prefill_products_holder'>".$this->generatePrefilledOption($cpbKeys, $key, $product_field_types)."</select></td>";

                echo "<td class = 'prefill_qty'><input type = 'number' name = 'wdm_prefill_qty[]' min = '1' max = '".$maxVal."' class = 'prefill_qty_id' value = '".$key['product_qty']."' /></td>";
                echo "<td><a class='wdm_cpb_rem' href='#' id=''><img class='add_new_row_image' src='".$removeImage."' /></a>";
                if ($i == count($prefill_list) - 1) {
                    echo "<a class='wdm_cpb_add' href='#' id=''><img class='add_new_row_image' src='".$addImage."' /></a>";
                }
                echo "</td></tr>";

                $i++;
            } //end of for
        } // end if
    } // end of function

    /**
     * Function to Generate Checkbox for pre-filled Mandatory products fetched from
     * DB.
     * @param  array $cpbKeys array of product id bundled in CPB product
     * @param  array $prefilledProduct  prefilled product data
     * @return string html tags
     */
    public function generateMandatoryCheckbox($cpbKeys, $prefilledProduct, $product_field_types)
    {
        if (in_array($prefilledProduct['unique_prod_id'], $cpbKeys)) {
            if ($prefilledProduct['product_mandatory'] == 1) {
                return "<td><input type = 'checkbox' class = 'prefill_checkbox' name = 'prod_mandatory[]' value = '".$prefilledProduct['unique_prod_id']."' checked/></td>";
            } else {
                return "<td><input type = 'checkbox' class = 'prefill_checkbox' name = 'prod_mandatory[]' value = '0' /></td>";
            }
        } else {
            return "<td><input type = 'checkbox' class = 'prefill_checkbox' name = 'prod_mandatory[]' value = '0' /></td>";
        }
    }

    /**
     * Function to Generate dropdown options for pre-filled products fetched from DB
     * @param  array $cpbKeys array of product id bundled in CPB product
     * @param  type $prefilledProduct prefilled product data
     * @return string $productHolder html tags
     */
    public function generatePrefilledOption($cpbKeys, $prefilledProduct, $product_field_types)
    {
        $productHolder = "";

        foreach ($cpbKeys as $index => $value) {
            $product_id = isset($product_field_types[$value]['variation_id']) ? $product_field_types[$value]['variation_id'] : $product_field_types[$value]['product_id'];
            $parentId = isset($product_field_types[$value]['parent_hash']) ? $product_field_types[$value]['parent_hash'] : $product_field_types[$value]['product_id'];
            $product_data = $product_field_types[$value];

            if ($value == $prefilledProduct['unique_prod_id']) {
                $productHolder .= "<option value = '".$value."' data-id = '".$product_id."' data-parent-id = '".$parentId."' selected>".$product_data['text']."</option>";
            } else {
                $productHolder .= "<option value = '".$value."' data-id = '".$product_id."' data-parent-id = '".$parentId."'>".$product_data['text']."</option>";
            }
        }

        return $productHolder;
    }

    //public function wdm_add_extra_options_setting($options) {
    //
    //  $options['_per_product_pricing_active_cpbz'] = array(
    //      'id' => '_per_product_pricing_active_cpb',
    //      'wrapper_class' => 'show_if_wdm_bundle_product',
    //      'label' => __('Per Product Pricing', 'custom-product-boxes'),
    //      'description' => __('If enabled, the Custom product box will be priced per-item, based on the prices of the selected items.', 'custom-product-boxes'),
    //      'default' => (get_post_meta(get_the_ID(), '_per_product_pricing_active', true) == 'yes') ? 'yes' : 'no',
    //  );
    //
    //  $options['_product_base_pricing_cpb'] = array(
    //      'id' => '_product_base_pricing_cpb',
    //      'wrapper_class' => 'show_if_wdm_bundle_product',
    //      'label' => __('Product Base Pricing', 'custom-product-boxes'),
    //      'description' => __('If enabled, the price of the Custom product box will be taken into consideration for dynamic pricing feature.', 'custom-product-boxes'),
    //      'default' => (get_post_meta(get_the_ID(), '_product_base_pricing_active', true) == 'yes') ? 'yes' : 'no',
    //  );
    //
    //  return $options;
    //}

    /**
    * Filters the variation from the array of add-on products. i.e If parent product exist then removes the
    * child variation of that product. As we display all variations of parent product.
    */
    public function filterChildrensOfExistingParents($bundle_data)
    {
        if (!$bundle_data) {
            return $bundle_data;
        }

        foreach ($bundle_data as $key => $value) {
            if (isset($value['parent_hash']) && array_key_exists($value['parent_hash'], $bundle_data)) {
                unset($bundle_data[$key]);
            }
        }

        return $bundle_data;
    }


    /**
     * Saves the product meta box data in the post meta table in the database.
     * Removes the deleted products data from the CPB Product
     * If pre-filled products checked, handle the pre-filled products data.
     * @param  int $post_id CPB product id
     */
    public function woocommerceProcessProductMetaBundleFunc($post_id)
    {
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        // Process product data
        global $prefillManager;
        $noOfBundleItems = 0;
        $needShippingCount = 0;
        $bundle_data = array();
        // update_option( 'notify_outofstock', 'yes', 'no' );
        // save custom product data
        $product_val = array(
            '_wdm_product_grid'                 => '',
            '_wdm_product_item_grid'            => '',
            '_wdm_grid_field'                   => '',
            '_wdm_column_field'                 => '',
            '_wdm_item_field'                   => '',
            '_wdm_cpb_pricing_type_field'       => '',
            '_wdm_desktop_layout'               => '',
            '_wdm_gift_boxes_color'             => '',
            '_wdm_gift_bgcolor'                 => '',
        );

        $productkeys = array_replace($product_val, array_intersect_key($_POST, $product_val));
        $productkeys = array_filter($productkeys);
        $wdm_reg_price_field='';
        $wdm_sale_price_field='';
        foreach ($productkeys as $key => $value) {
            update_post_meta($post_id, $key, esc_attr($value));
        }

        // save product price
        if (isset($_POST['_regular_price'])) {
            $wdm_reg_price_field    = wc_clean($_POST['_regular_price']);
        }

        // if (empty($_POST['_regular_price'])) {
        //     $wdm_reg_price_field    = 0;
        // }

        if (isset($_POST['_sale_price'])) {
            $wdm_sale_price_field=wc_clean($_POST['_sale_price']);
        }

        if (isset($_POST['_sale_price_dates_from'])) {
            $sale_price_dates_from = "";
            update_post_meta($post_id, '_sale_price_dates_from', $sale_price_dates_from);
        }
        
        if (isset($_POST['_sale_price_dates_to'])) {
            $sale_price_dates_from = "";
            update_post_meta($post_id, '_sale_price_dates_to', $sale_price_dates_from);
        }

        update_post_meta($post_id, '_regular_price', '' === $wdm_reg_price_field ? '' : wc_format_decimal($wdm_reg_price_field));

        update_post_meta($post_id, '_sale_price', '' === $wdm_sale_price_field ? '' : wc_format_decimal($wdm_sale_price_field));

        if ('' !== $wdm_sale_price_field) {
            update_post_meta($post_id, '_price', wc_format_decimal($wdm_sale_price_field));
        } else {
            update_post_meta($post_id, '_price', wc_format_decimal($wdm_reg_price_field));
        }

        $wdm_product_column = $productkeys['_wdm_cpb_pricing_type_field'];
        switch ($wdm_product_column) {
            case 'wdm-cpb-fixed-pp':
                update_post_meta($post_id, '_per_product_pricing_active', 'no');
                update_post_meta($post_id, '_product_base_pricing_active', 'yes');
                break;

            case 'wdm-cpb-dynamic-pp-base':
                update_post_meta($post_id, '_per_product_pricing_active', 'yes');
                update_post_meta($post_id, '_product_base_pricing_active', 'yes');
                break;

            case 'wdm-cpb-dynamic-pp-nobase':
                update_post_meta($post_id, '_per_product_pricing_active', 'yes');
                update_post_meta($post_id, '_product_base_pricing_active', 'no');
                update_post_meta($post_id, '_price', 0);
                break;
        }

        update_post_meta($post_id, '_sold_individually', "no");
        update_post_meta($post_id, '_per_product_shipping_active', 'no');
        update_post_meta($post_id, '_virtual', 'no');
        update_post_meta($post_id, '_weight', stripslashes($_POST['_weight']));
        update_post_meta($post_id, '_length', stripslashes($_POST['_length']));
        update_post_meta($post_id, '_width', stripslashes($_POST['_width']));
        update_post_meta($post_id, '_height', stripslashes($_POST['_height']));

        // Storing subscription data
        $this->saveSubscriptionData($post_id);


        // Storing Bundle data
        $wdm_bundle_data = array();

        if (isset($_POST['product_field_type']) && isset($_POST['add_onProducts'])) {
            $addOnProducts = json_decode(stripslashes($_POST['add_onProducts']));
            $wdm_bundle_data =$this->convertIntoCPBArray($addOnProducts);
        }

        $bundle_data = unlinkDeletedProducts($wdm_bundle_data, $post_id);

        $bundle_data = $this->filterChildrensOfExistingParents($bundle_data);


        /*
            Will decide if all the products in the bundle has no shipping then make the CPB product as no shipping.
            This will add a meta to the custom box 
        */
        if ($needShippingCount > 0 && $needShippingCount <= $noOfBundleItems) {
            update_post_meta($post_id, 'wdm_need_shipping', true);
        } else {
            update_post_meta($post_id, 'wdm_need_shipping', false);
        }

        update_post_meta($post_id, '_bundle_data', $bundle_data);

        //Allow the purchase of partially-filled boxes
        if (isset($_POST['wdm_boxes_selection'])) {
            update_post_meta($post_id, 'wdm_boxes_selection', $_POST['wdm_boxes_selection']);
        } else {
            update_post_meta($post_id, 'wdm_boxes_selection', 'no');
        }

        //Sort Products by Date
        if (isset($_POST['wdm_order_by_date'])) {
            update_post_meta($post_id, 'wdm_order_by_date', $_POST['wdm_order_by_date']);
        } else {
            update_post_meta($post_id, 'wdm_order_by_date', 'no');
        }
        //Allow Scroll Lock
        if (isset($_POST['wdm_disable_scroll'])) {
            update_post_meta($post_id, 'wdm_disable_scroll', $_POST['wdm_disable_scroll']);
        } else {
            update_post_meta($post_id, 'wdm_disable_scroll', 'no');
        }

        //Allow pre-filled boxes
        if (isset($_POST['wdm_prefilled_box'])) {
            update_post_meta($post_id, 'wdm_prefilled_box', $_POST['wdm_prefilled_box']);
            $this->processPrefilledProductsData($post_id, $wdm_bundle_data);
        } else {
            $prefillManager->deletePrefilledProducts($post_id);
            update_post_meta($post_id, 'wdm_prefilled_box', 'no');
        }
        //Allows user to remove mandatory products only if they run out of stock
        if (isset($_POST['wdm_swap_products'])) {
            update_post_meta($post_id, 'wdm_swap_products', $_POST['wdm_swap_products']);
        } else {
            update_post_meta($post_id, 'wdm_swap_products', 'no');
        }

        //Enable gift message field.
        $enableGiftMessage = isset($_POST['_wdm_enable_gift_message']) ? $_POST['_wdm_enable_gift_message'] : 'no';
        $giftMessageLabel = isset($_POST['_wdm_gift_message_label']) ? $_POST['_wdm_gift_message_label'] : '';

        update_post_meta($post_id, '_wdm_enable_gift_message', $enableGiftMessage);

        if ($enableGiftMessage == 'yes') {
            $giftMessageLabel = empty($giftMessageLabel) ? __("Gift Message", 'custom-product-boxes') : $giftMessageLabel;
            update_post_meta($post_id, '_wdm_gift_message_label', $giftMessageLabel);
        } else {
            update_post_meta($post_id, '_wdm_gift_message_label', '');
        }
    }

    public function saveSubscriptionData($post_id)
    {
        // if our nonce isn't there, or we can't verify it
        if (!isset($_POST['wdm_cpb_nonce']) || !wp_verify_nonce($_POST['wdm_cpb_nonce'], 'wdm_cpb_nonce')) {
            return;
        }

        // if current user can't edit this post
        if (!current_user_can('edit_post', $post_id)) {
            return;
        }

        //subscription code
        if (isset($_POST['wdm_enable_product_subscription']) && sanitize_text_field($_POST['_subscription_price']) != 0) {
            wp_set_object_terms($post_id, 'subscription', 'wdm_bundle_product');
            update_post_meta($post_id, '_subscription_price', sanitize_text_field($_POST['_subscription_price']));
            update_post_meta($post_id, '_subscription_sign_up_fee', sanitize_text_field($_POST['_subscription_sign_up_fee']));
            update_post_meta($post_id, '_subscription_period', sanitize_text_field($_POST['_subscription_period']));
            update_post_meta($post_id, '_subscription_period_interval', sanitize_text_field($_POST['_subscription_period_interval']));
            update_post_meta($post_id, '_subscription_length', sanitize_text_field($_POST['_subscription_length']));
            update_post_meta($post_id, '_subscription_trial_period', sanitize_text_field($_POST['_subscription_trial_period']));
            update_post_meta($post_id, '_subscription_trial_length', sanitize_text_field($_POST['_subscription_trial_length']));
            update_post_meta($post_id, 'wdm_enable_product_subscription', 'yes');
        } else {
            wp_remove_object_terms($post_id, 'subscription', 'wdm_bundle_product');
            delete_post_meta($post_id, '_subscription_price');
            delete_post_meta($post_id, '_subscription_sign_up_fee');
            delete_post_meta($post_id, '_subscription_period');
            delete_post_meta($post_id, '_subscription_period_interval');
            delete_post_meta($post_id, '_subscription_length');
            delete_post_meta($post_id, '_subscription_trial_period');
            delete_post_meta($post_id, '_subscription_trial_length');
            update_post_meta($post_id, 'wdm_enable_product_subscription', 'no');
        }
    }

    public function convertIntoCPBArray($bundleData)
    {
        $convertedData = array();
        if (empty($bundleData)) {
            return $bundleData;
        }

        foreach ($bundleData as $bundleObject) {
            $convertedBundle = $this->getVariationData($bundleObject);
            $convertedData[$bundleObject->id] = $convertedBundle;
        }
        
        return $convertedData;
    }


    public function getVariationData($bundleObject)
    {
        $convertedBundle=(array)$bundleObject;

        if ($convertedBundle['product_type'] == "variable") {
            $excluded_products = array(md5($bundleObject->product_id));
            $product = wc_get_product($bundleObject->product_id);
            $variationData = (array) $convertedBundle['childrens'];
            $convertedBundle['childrens'] = $this->formatVariationTitle($variationData);
        }

        return $convertedBundle;
    }

    public function formatVariationTitle($variationData)
    {
        if (!is_array($variationData) || empty($variationData)) {
            return $variationData;
        }

        $newVariationData = array();

        foreach ($variationData as $key => $value) {
            $variationDataArray = (array) $value;
            $newVariationData[$key] = $variationDataArray;
            $newVariationData[$key]['id'] = $key;
            if (isset($variationDataArray['text_name'])) {
                $newVariationData[$key]['text'] = $variationDataArray['text_name'];
                unset($newVariationData[$key]['text_name']);
            }
        }

        return $newVariationData;
    }

    /**
     * Processes the prefilled data to be stored in DB
     * Update the pre-filled products data in DB with the pre-filled products
     * present in the current selection;.
     * @param  int $postId CPB product id
     */
    public function processPrefilledProductsData($postId, $bundle_data)
    {
        global $prefillManager;
        $bundleData = getPrefillBundleForVariableProduct($bundle_data);
        $prefillProducts = isset($_POST['wdm_cpb_products']) ? $_POST['wdm_cpb_products'] : array();
        $prefillQty = isset($_POST['wdm_prefill_qty']) ? $_POST['wdm_prefill_qty'] : array();
        $prefillMandatory = isset($_POST['prod_mandatory']) ? $_POST['prod_mandatory'] : array();

        $newPrefillProducts = array();
        $newPrefillQty = array();
        $newPrefillMandatory = array();
        $pos = 0;

        foreach ($prefillProducts as $key => $value) { // filters unique product so that duplicate records do not get added
            $index = array_search($value, $newPrefillProducts);
            if (false === $index) {
                $newPrefillProducts[$pos] = $value;
                $newPrefillQty[$pos] = $prefillQty[$key];
                $newPrefillMandatory[$pos] = isset($prefillMandatory[$key]) ? $prefillMandatory[$key] : "";
                $pos++;
            } elseif (false !== $index && $newPrefillQty[$index] !== $prefillQty[$key]) {
                $newPrefillProducts[$pos] = $value;
                $newPrefillQty[$pos] = $prefillQty[$key];
                $newPrefillMandatory[$pos] = $prefillMandatory[$key];
                $pos++;
            }
        }
        print_r($newPrefillProducts);

        $prefillCheckbox = array();
        foreach ($newPrefillProducts as $key => $value) {
            if (in_array($value, $newPrefillMandatory)) {
                $prefillCheckbox[$key] = 1;
            } else {
                $prefillCheckbox[$key] = 0;
            }
        }

        if ($newPrefillProducts && $newPrefillQty) {
            $prefillManager->savePrefilledProducts($postId, $newPrefillProducts, $newPrefillQty, $prefillCheckbox, $bundleData);
        }
    }
}
